﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace PropertyGridPlus
{
  public partial class PropertyGridPlus : PropertyGrid
  {
    public PropertyGridPlus()
    {
      InitializeComponent();
      toolStripMenuItemCopySelected.Click += ToolStripMenuItemCopySelected_Click;
      toolStripMenuItemOneLevelWithIndent.Click += ToolStripMenuItemOneLevelWithIndent_Click;
      toolStripMenuItemOneLevelWithoutIndent.Click += ToolStripMenuItemOneLevelWithoutIndent_Click;
      toolStripMenuItemCopyAllWithIndent.Click += ToolStripMenuItemCopyAllWithIndent_Click;
      toolStripMenuItemCopyAllWithoutIndent.Click += ToolStripMenuItemCopyAllWithoutIndent_Click;
      toolStripMenuItemCopyMoreWithIndent.Click += ToolStripMenuItemCopyMoreWithIndent_Click;
      toolStripMenuItemCopyMoreWithoutIndent.Click += ToolStripMenuItemCopyMoreWithoutIndent_Click;
    }

    private void ToolStripMenuItemOneLevelWithIndent_Click(object sender, EventArgs e)
    {
      string text = CreateText(SelectedGridItem, true, 1);
      Clipboard.SetText(text);
    }
    private void ToolStripMenuItemOneLevelWithoutIndent_Click(object sender, EventArgs e)
    {
      string text = CreateText(SelectedGridItem, true, 1);
      Clipboard.SetText(text);
    }
    private void ToolStripMenuItemCopyAllWithIndent_Click(object sender, EventArgs e)
    {
      string text = CreateText(SelectedGridItem, true);
      Clipboard.SetText(text);
    }
    private void ToolStripMenuItemCopyAllWithoutIndent_Click(object sender, EventArgs e)
    {
      string text = CreateText(SelectedGridItem, false);
      Clipboard.SetText(text);
    }
    private void ToolStripMenuItemCopyMoreWithIndent_Click(object sender, EventArgs e)
    {
      int depthVal = 0;
      int.TryParse(toolStripComboBoxLevelSetting.Text, out depthVal);
      string text = CreateText(SelectedGridItem, true, depthVal);
      Clipboard.SetText(text);
    }

    private void ToolStripMenuItemCopyMoreWithoutIndent_Click(object sender, EventArgs e)
    {
      int depthVal = 0;
      int.TryParse(toolStripComboBoxLevelSettingWithoutIndent.Text, out depthVal);
      string text = CreateText(SelectedGridItem, false, depthVal);
      Clipboard.SetText(text);
    }

    private void ToolStripMenuItemCopySelected_Click(object sender, EventArgs e)
    {
//      string text = CreateText(SelectedGridItem, false, 1);
      string text = CreateText(SelectedGridItem, true);
      Clipboard.SetText(text);
    }
    /// <summary>
    /// 選択した項目の内容をテキストに出力します。
    /// </summary>
    /// <param name="item">選択した項目</param>
    /// <param name="useIndent">インデント処理するか</param>
    /// <param name="limitDepth">子アイテムを読み込む深さ</param>
    /// <param name="depth">再起処理用のカウンタ(内部用)</param>
    /// <returns></returns>
    public string CreateText(GridItem item, bool useIndent, int limitDepth = int.MaxValue, int depth = 0)
    {
      string ret = "";
      if (useIndent)
      {
        int depth_max;
        depth_max = getDepthFromGridItem(item, limitDepth);
        ret = createTextWithIndent(item, limitDepth, depth_max);

      }
      else
      {
        ret = createTextWithoutIndent(item, limitDepth);
      }

      return ret;
    }

    /// <summary>
    /// 指定した深さ以内で最も深い深さを取得します。
    /// </summary>
    /// <param name="item">PropertyGridのアイテム</param>
    /// <param name="limitDepth">最大深さ</param>
    /// <param name="currentDepth">再帰処理用の内部カウンタ。呼び出す際は指定しない。</param>
    /// <returns></returns>
    private int getDepthFromGridItem(GridItem item, int limitDepth, int currentDepth = 0)
    {
      //指定した深さ以上になっていればツリーの展開を中断し、現在の深さをそのまま返す。
      if (currentDepth >= limitDepth)
      {
        return currentDepth;
      }
      //サブツリーの展開
      int retDepth = currentDepth;
      foreach (var child in item.GridItems)
      {
        int childDepth = getDepthFromGridItem(child as GridItem, limitDepth, currentDepth + 1);
        //サブツリーの中で最も深い深さを取得する。
        if (retDepth < childDepth)
          retDepth = childDepth;
      }
      return retDepth;
    }

    /// <summary>
    /// GridItemからツリーのコピーテキストを取得します。
    /// </summary>
    /// <param name="item">PropertyGridのアイテム</param>
    /// <param name="limitDepth">最大の深さ</param>
    /// <param name="maxDepth">最も深いノードの深さ(getDepthFromGridItemで取得する)</param>
    /// <param name="currentDepth">再帰処理用の内部カウンタ。呼び出す際は指定しない。</param>
    /// <returns></returns>
    private string createTextWithIndent(GridItem item, int limitDepth, int maxDepth, int currentDepth = 0)
    {
      StringBuilder ret = new StringBuilder();
      //インデントのため、タブを現在の深さ分挿入する。
      ret.Append(new string('\t', currentDepth));
      //子アイテムがあるときの記号。自由に設定してください。
      string expandSign = "";
      if (item.GridItems.Count > 0)
        expandSign = "▼";
      //プロパティ名の追加
      ret.Append($"{expandSign}{item.Label.TrimStart('\t')}");
      //プロパティ名と値の間のスペースを追加
      ret.Append($"{new string('\t', maxDepth - currentDepth + 1)}");
      //値の出力
      ret.AppendLine($"{item.Value}");
      //サブツリーの展開
      if (currentDepth < limitDepth)
      {
        foreach (var child in item.GridItems)
        {
          ret.Append(createTextWithIndent(child as GridItem, limitDepth, maxDepth, currentDepth + 1));
        }
      }
      return ret.ToString();
    }

    /// <summary>
    /// GridItemからツリーのコピーテキストを取得します。(インデント無し版)
    /// </summary>
    /// <param name="item">PropertyGridのアイテム</param>
    /// <param name="limitDepth">最大の深さ</param>
    /// <param name="maxDepth">最も深いノードの深さ(getDepthFromGridItemで取得する)</param>
    /// <param name="currentDepth">再帰処理用の内部カウンタ。呼び出す際は指定しない。</param>
    /// <returns></returns>
    private string createTextWithoutIndent(GridItem item, int limitDepth, int currentDepth = 0)
    {
      StringBuilder ret = new StringBuilder();
      ret.AppendLine($"{item.Label}\t{item.Value}");
      if (currentDepth < limitDepth)
      {
        foreach (var child in item.GridItems)
        {
          ret.Append(createTextWithoutIndent(child as GridItem, limitDepth, currentDepth + 1));
        }
      }
      return ret.ToString();
    }


    //private string _createTextWithoutIndent(GridItem item, int limitDepth, int currentDepth = 0)
    //{
    //  StringBuilder ret = new StringBuilder();
    //  ret.AppendLine($"{item.Label}\t{item.Value}");//タブ区切り
    //  if (currentDepth < limitDepth)
    //  {
    //    foreach (var child in item.GridItems)
    //    {
    //      ret.Append(_createTextWithoutIndent(child as GridItem, limitDepth, currentDepth + 1));
    //    }
    //  }
    //  return ret.ToString();
    //}


    protected override void OnPaint(PaintEventArgs pe)
    {
      base.OnPaint(pe);
    }
  }
}
